/** preferences.js
 * 
 */
'use strict'

const {Gio, Gtk} = imports.gi;

const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();

const gettextDomain = Me.metadata['gettext-domain'];

const UI_FOLDER = Me.dir.get_child('ui').get_path();

var Prefs = class {

    /**
     * Class contructor
     */
    constructor() {
        this._settings = ExtensionUtils.getSettings();
        this._builder = new Gtk.Builder();
        this._builder.set_translation_domain(gettextDomain);
    }

    /**
     * Fill prefs window.
     *
     * @param {AdwWindow} window 
     *
     * @returns {void}
     */
    fillPrefsWindow(window) {
        this._builder.add_from_file(UI_FOLDER + '/prefs.ui');

        const generalPrefs = this._builder.get_object('general_prefs');
        const thresholdsPrefs = this._builder.get_object('thresholds_prefs');
        const experimentalPrefs = this._builder.get_object('experimental_prefs');

        window.add(generalPrefs);
        window.add(thresholdsPrefs);
        window.add(experimentalPrefs);

        this._connectSignals(window);

        window.search_enabled = true;
    }

    /**
     * Connect widges signals.
     * 
     * @param {Gtk.Window} window
     */
    _connectSignals(window) {
        this._bindDropdown('indicator_mode', 'indicator-mode');
        this._bindSwitch('color_mode', 'color-mode');
        this._bindSwitch('show_values', 'show-current-values');
        this._bindSwitch('show_notifications', 'show-notifications');
        this._bindSwitch('debug_mode', 'debug-mode');
        this._bindSwitch('model_verification', 'model-verification');

        this._bindSpinbutton('start_bat0', 'start-bat0');
        this._bindSpinbutton('end_bat0', 'end-bat0');
        this._bindSpinbutton('start_bat1', 'start-bat1');
        this._bindSpinbutton('end_bat1', 'end-bat1');

        // Rules
        const startBat0 = this._builder.get_object('start_bat0');
        const endBat0 = this._builder.get_object('end_bat0');
        const startBa1 = this._builder.get_object('start_bat1');
        const endBat1 = this._builder.get_object('end_bat1');

        this._settings.connect('changed::start-bat0', () => {
            if (startBat0.value >= endBat0.value) {
                endBat0.value = startBat0.value + 1;
            }
        });
        this._settings.connect('changed::end-bat0', () => {
            if (startBat0.value >= endBat0.value) {
                startBat0.value = endBat0.value - 1;
            }
        });
        this._settings.connect('changed::start-bat1', () => {
            if (startBa1.value >= endBat1.value) {
                endBat1.value = startBa1.value + 1;
            }
        });
        this._settings.connect('changed::end-bat1', () => {
            if (startBa1.value >= endBat1.value) {
                startBa1.value = endBat1.value - 1;
            }
        });
    }

    /**
     * Bind GtkSwitch.
     * 
     * @param {string} objectName Object id
     * @param {string} settingKey Settings key
     */
    _bindSwitch(objectName, settingKey) {
        const object = this._builder.get_object(objectName);
        this._settings.bind(settingKey, object, 'state', Gio.SettingsBindFlags.DEFAULT);
    }

    /**
     * Bind GtkDropDown.
     * 
     * @param {string} objectName Object id
     * @param {string} settingKey Settings key
     */
    _bindDropdown(objectName, settingKey) {
        const object = this._builder.get_object(objectName);
        object.selected = this._settings.get_enum(settingKey);
        object.connect('notify::selected-item', () => {
            this._settings.set_enum(settingKey, object.selected);
        });
    }

    /**
     * Bind GtkSpinbutton.
     * 
     * @param {string} objectName Object id
     * @param {string} settingKey Settings key
     */
    _bindSpinbutton(objectName, settingKey) {
        const object = this._builder.get_object(objectName);
        this._settings.bind(settingKey, object, 'value', Gio.SettingsBindFlags.DEFAULT);
    }
}
